class Dhatu::BlogPost < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "blog_posts"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validates :title, presence: true, length: {minimum: 5, maximum: 256}, allow_blank: false
  validates :slug, uniqueness: true, presence: true, length: {minimum: 5, maximum: 64}, allow_blank: false
  validates :author, length: {minimum: 3, maximum: 250}, allow_blank: true
  validates :description, presence: true
  validates :posted_at, presence: true
  validates :short_description, presence: true, length: {minimum: 10, maximum: 500}, allow_blank: false

  validates :category, presence: true
  
  # Associations
  belongs_to :category
  has_one :cover_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::CoverImage"
  has_many :gallery_images, :as => :imageable, :dependent => :destroy, :class_name => "Image::GalleryImage"

  # Meta Tag Associations
  has_one :og_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::OgImage"
  has_many :meta_tags, :as => :meta_taggable, :dependent => :destroy, :class_name => "Dhatu::MetaTag"

  # Serializers
  serialize :tags, Hash
  
  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda { |query| where("LOWER(title) LIKE LOWER('%#{query}%') OR\
                                        LOWER(author) LIKE LOWER('%#{query}%') OR\
                                        LOWER(description) LIKE LOWER('%#{query}%')") }

  scope :upcoming, lambda { where("starts_at >= ?", Time.now) }
  scope :past, lambda { where("starts_at < ?", Time.now) }

  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:title].to_s.strip.blank?

    category = Dhatu::Category.find_by_name(hsh[:category].to_s.strip)

    unless category
      puts "Category '#{hsh[:category].to_s.strip}' not found".red
      return
    end

    blog_post = Dhatu::BlogPost.where("title = ? AND category_id = ?", hsh[:title].to_s.strip, category.id).first || Dhatu::BlogPost.new
    
    blog_post.title = hsh[:title].to_s.strip
    blog_post.slug = blog_post.title.parameterize[0..64] if blog_post.title
    blog_post.author = hsh[:author].to_s.strip
    blog_post.short_description = hsh[:short_description].to_s.strip
    blog_post.description = hsh[:description].to_s.strip
    blog_post.posted_at = hsh[:posted_at].to_s.strip

    blog_post.category = category

    blog_post.featured = ["true", "t","1","yes","y"].include?(hsh[:featured].to_s.downcase.strip)
    blog_post.status = hsh[:status].to_s.strip.blank? ? PUBLISHED : hsh[:status].to_s.strip

    if blog_post.valid?
      begin
        blog_post.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving blog_post: #{blog_post.title}"
      details = "Error! #{blog_post.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end
  
  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------

  def display_name
    "#{title_was}"
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end
  
end
