class Dhatu::Branch < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "branches"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validates :title, presence: true, length: {maximum: 250}

  validates :address_1, length: {maximum: 250}, allow_blank: true
  validates :address_2, length: {maximum: 250}, allow_blank: true
  validates :address_3, length: {maximum: 250}, allow_blank: true

  # validate_email :email, presence: false, allow_blank: false
  validates :landline, length: {maximum: 250}, allow_blank: true
  validates :fax, length: {maximum: 250}, allow_blank: true
  validates :mobile, length: {maximum: 250}, allow_blank: true

  validates :facebook, format: { with: URI.regexp }, if: Proc.new { |ci| ci.facebook.present? }
  validates :twitter, format: { with: URI.regexp }, if: Proc.new { |ci| ci.twitter.present? }
  validates :google_plus, format: { with: URI.regexp }, if: Proc.new { |ci| ci.google_plus.present? }
  validates :linked_in, format: { with: URI.regexp }, if: Proc.new { |ci| ci.linked_in.present? }
  validates :youtube, format: { with: URI.regexp }, if: Proc.new { |ci| ci.youtube.present? }
  validates :instagram, format: { with: URI.regexp }, if: Proc.new { |ci| ci.instagram.present? }
  validates :tumblr, format: { with: URI.regexp }, if: Proc.new { |ci| ci.tumblr.present? }
  validates :pinterest, format: { with: URI.regexp }, if: Proc.new { |ci| ci.pinterest.present? }
  validates :blog, format: { with: URI.regexp }, if: Proc.new { |ci| ci.blog.present? }

  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda {|query| where("
                                        LOWER(title) LIKE LOWER('%#{query}%') OR\
                                        LOWER(address_1) LIKE LOWER('%#{query}%') OR\
                                        LOWER(address_2) LIKE LOWER('%#{query}%') OR\
                                        LOWER(address_3) LIKE LOWER('%#{query}%') OR\
                                        LOWER(email) LIKE LOWER('%#{query}%') OR\
                                        LOWER(landline) LIKE LOWER('%#{query}%') OR\
                                        LOWER(fax) LIKE LOWER('%#{query}%') OR\
                                        LOWER(mobile) LIKE LOWER('%#{query}%') OR\
                                        LOWER(facebook) LIKE LOWER('%#{query}%') OR\
                                        LOWER(twitter) LIKE LOWER('%#{query}%') OR\
                                        LOWER(google_plus) LIKE LOWER('%#{query}%') OR\
                                        LOWER(linked_in) LIKE LOWER('%#{query}%') OR\
                                        LOWER(youtube) LIKE LOWER('%#{query}%') OR\
                                        LOWER(instagram) LIKE LOWER('%#{query}%') OR\
                                        LOWER(tumblr) LIKE LOWER('%#{query}%') OR\
                                        LOWER(pinterest) LIKE LOWER('%#{query}%') OR\
                                        LOWER(blog) LIKE LOWER('%#{query}%')")
  }
  
  scope :upcoming, lambda { where("created_at >= ?", Time.now) }
  scope :past, lambda { where("created_at < ?", Time.now) }
  scope :main_branch, lambda { where("main_branch is TRUE")}

  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:title].to_s.strip.blank?

    branch = Dhatu::Branch.find_by_title(hsh[:title].to_s.strip) || Dhatu::Branch.new
    
    branch.title = hsh[:title].to_s.strip
    branch.address_1 = hsh[:address_1].to_s.strip
    branch.address_2 = hsh[:address_2].to_s.strip
    branch.address_3 = hsh[:address_3].to_s.strip
    branch.email = hsh[:email].to_s.strip
    branch.landline = hsh[:landline].to_s.strip
    branch.fax = hsh[:fax].to_s.strip
    branch.mobile = hsh[:mobile].to_s.strip

    branch.facebook = hsh[:facebook].to_s.strip
    branch.twitter = hsh[:twitter].to_s.strip
    branch.google_plus = hsh[:google_plus].to_s.strip
    branch.linked_in = hsh[:linked_in].to_s.strip
    branch.youtube = hsh[:youtube].to_s.strip
    branch.instagram = hsh[:instagram].to_s.strip
    branch.tumblr = hsh[:tumblr].to_s.strip
    branch.pinterest = hsh[:pinterest].to_s.strip
    branch.blog = hsh[:blog].to_s.strip
    branch.status = hsh[:status].to_s.strip.blank? ? PUBLISHED : hsh[:status].to_s.strip

    branch.featured = ["true", "t","1","yes","y"].include?(hsh[:featured].to_s.downcase.strip)
    branch.main_branch = ["true", "t","1","yes","y"].include?(hsh[:main_branch].to_s.downcase.strip)

    if branch.valid?
      begin
        branch.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving branch: #{branch.title}"
      details = "Error! #{branch.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end

  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------
  def to_param
    "#{id}-#{title.parameterize[0..32]}"
  end

  def display_name
    "#{title_was}"
  end

  def display_address
    [self.address_1, self.address_2, self.address_3].compact.uniq.join(", ")
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end

end