class Dhatu::Event < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "events"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validates :title, presence: true, length: {minimum: 5, maximum: 500}, allow_blank: false
  validates :venue, length: {minimum: 3, maximum: 250}, allow_blank: true
  validates :description, presence: true

  # Associations
  belongs_to :category, optional: true
  has_one :cover_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::CoverImage"
  has_many :gallery_images, :as => :imageable, :dependent => :destroy, :class_name => "Image::GalleryImage"

  # Meta Tag Associations
  has_one :og_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::OgImage"
  has_many :meta_tags, :as => :meta_taggable, :dependent => :destroy, :class_name => "Dhatu::MetaTag"
  
  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda { |query| where("LOWER(title) LIKE LOWER('%#{query}%') OR\
                                        LOWER(venue) LIKE LOWER('%#{query}%') OR\
                                        LOWER(description) LIKE LOWER('%#{query}%')") }

  scope :upcoming, lambda { where("starts_at >= ?", Time.now) }
  scope :past, lambda { where("starts_at < ?", Time.now) }
  
  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:title].to_s.strip.blank?

    event = Dhatu::Event.find_by_title(hsh[:title].to_s.strip) || Dhatu::Event.new
    event.title = hsh[:title].to_s.strip
    event.venue = hsh[:venue].to_s.strip
    event.description = hsh[:description].to_s.strip
    event.date = hsh[:date].to_s.strip
    event.starts_at = hsh[:starts_at].to_s.strip
    event.ends_at = hsh[:ends_at].to_s.strip
    
    # event.category = Dhatu::Category.find_by_name(hsh[:category])
    event.status = hsh[:status].to_s.strip || PUBLISHED
    event.featured = hsh[:featured].to_s.strip == "true"
    event.priority = hsh[:priority].to_s.strip || 1

    if event.valid?
      begin
        event.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving event: #{event.title}"
      details = "Error! #{event.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end

  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------

  def to_param
    "#{id}-#{title.parameterize[0..32]}"
  end

  def display_name
    "#{title_was}"
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end
  
end
