class Dhatu::Project < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "projects"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validate_string :name, mandatory: true, min_length: 2, format: /.*/i
  validate_string :short_description, mandatory: false, format: /.*/i
  validate_string :permalink, mandatory: true, format: /.*/i, min_length: 4, max_length: 128
  
  # Associations
  belongs_to :category, optional: true
  has_one :cover_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::CoverImage"
  has_many :gallery_images, :as => :imageable, :dependent => :destroy, :class_name => "Image::GalleryImage"

  # Meta Tag Associations
  has_one :og_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::OgImage"
  has_many :meta_tags, :as => :meta_taggable, :dependent => :destroy, :class_name => "Dhatu::MetaTag"

  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda {|query| where("LOWER(name) LIKE LOWER('%#{query}%') OR\
                                        LOWER(short_description) LIKE LOWER('%#{query}%') OR\
                                        LOWER(permalink) LIKE LOWER('%#{query}%') OR\
                                        LOWER(description) LIKE LOWER('%#{query}%')")
  }
  
  scope :filter_by_category, lambda { |c| where("category_id = ?", (c.is_a? Dhatu::Category ? c.id : c)) }

  scope :upcoming, lambda { where("created_at >= ?", Time.now) }
  scope :past, lambda { where("created_at < ?", Time.now) }

  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:name].to_s.strip.blank?

    project = Dhatu::Project.find_by_name(hsh[:name].to_s.strip) || Dhatu::Project.new
    project.name = hsh[:name].to_s.strip
    project.short_description = hsh[:short_description].to_s.strip
    project.description = hsh[:description].to_s.strip
    project.client = hsh[:client].to_s.strip
    project.category = Dhatu::Category.find_by_name(hsh[:category].to_s.strip)
    project.status = hsh[:status].to_s.strip.blank? ? PUBLISHED : hsh[:status].to_s.strip
    project.featured = hsh[:featured].to_s.strip || true
    project.priority = hsh[:priority].to_s.strip || 1

    project.generate_permalink

    if project.valid?
      begin
        project.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving project: #{project.name}"
      details = "Error! #{project.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end

  # Image Configuration
  # -------------
  def self.image_configuration
    {
      "Image::CoverImage" => {
        max_upload_limit: 10485760,
        min_upload_limit: 1,
        resolutions: [800, 400],
        form_upload_image_label: "Upload a new Image",
        form_title: "Upload an Image",
        form_sub_title: "Please read the instructions below:",
        form_instructions: [
          "the filename should be in .jpg / .jpeg or .png format",
          "the image resolutions should be <strong>800 x 400 Pixels</strong>",
          "the file size should be greater than 100 Kb and or lesser than <strong>10 MB</strong>",
          "Note: most cameras and camera phones will produce images bigger than this"
        ]
      },
      "Image::GalleryImage" => {
        max_upload_limit: 10485760,
        min_upload_limit: 1,
        resolutions: [800, 400],
        form_upload_image_label: "Upload a new Image",
        form_title: "Upload an Image",
        form_sub_title: "Please read the instructions below:",
        form_instructions: [
          "the filename should be in .jpg / .jpeg or .png format",
          "the image resolutions should be <strong>800 x 400 Pixels</strong>",
          "the file size should be greater than 100 Kb and or lesser than <strong>10 MB</strong>",
          "Note: most cameras and camera phones will produce images bigger than this"
        ]
      }  
    }
    
  end

  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------
  def to_param
    "#{id}-#{name.parameterize[0..32]}"
  end

  def generate_permalink
    self.permalink = self.name.parameterize[0..32]
  end

  def display_name
    "#{name_was}"
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end

end