class Dhatu::Promotion < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "promotions"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validates :title, presence: true, length: {minimum: 5, maximum: 256}, allow_blank: false
  validates :code, length: {minimum: 5, maximum: 64}, allow_blank: true
  validates :short_description, length: {minimum: 3, maximum: 512}, allow_blank: true
  
  # Associations
  has_many :promotion_attributes
  has_many :promotion_enquiries
  has_one :cover_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::CoverImage"
  
  # Callbacks
  after_create :generate_and_save_code

  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda { |query| where("LOWER(title) LIKE LOWER('%#{query}%') OR\
                                        LOWER(code) LIKE LOWER('%#{query}%') OR\
                                        LOWER(short_description) LIKE LOWER('%#{query}%')")}

  scope :upcoming, lambda { where("starts_at >= ?", Time.now) }
  scope :past, lambda { where("starts_at < ?", Time.now) }
  
  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:title].to_s.strip.blank?

    promotion = Dhatu::Promotion.find_by_title(hsh[:title].to_s.strip) || Dhatu::Promotion.new

    promotion.title = hsh[:title].to_s.strip
    promotion.code = hsh[:code].to_s.strip
    promotion.short_description = hsh[:short_description].to_s.strip

    promotion.display_name = ["true", "t","1","yes","y"].include?(hsh[:display_name].to_s.downcase.strip)
    promotion.display_email = ["true", "t","1","yes","y"].include?(hsh[:display_email].to_s.downcase.strip)
    promotion.display_phone = ["true", "t","1","yes","y"].include?(hsh[:display_phone].to_s.downcase.strip)

    promotion.name_mandatory = ["true", "t","1","yes","y"].include?(hsh[:name_mandatory].to_s.downcase.strip)
    promotion.email_mandatory = ["true", "t","1","yes","y"].include?(hsh[:email_mandatory].to_s.downcase.strip)
    promotion.phone_mandatory = ["true", "t","1","yes","y"].include?(hsh[:phone_mandatory].to_s.downcase.strip)
    
    promotion.featured = ["true", "t","1","yes","y"].include?(hsh[:featured].to_s.downcase.strip)
    promotion.status = hsh[:status].to_s.strip.blank? ? PUBLISHED : hsh[:status].to_s.strip
    promotion.priority = hsh[:priority].to_s.strip.blank? ? 1000 : hsh[:priority].to_s.strip

    if promotion.valid?
      begin
        promotion.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving promotion: #{promotion.title}"
      details = "Error! #{promotion.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end

  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------
  def to_param
    "#{id}-#{title.parameterize[0..32]}"
  end

  def display_name
    "#{title_was}"
  end

  def promotion_attributes_with_values
    hsh = {}
    self.promotion_attributes.published.where("mandatory is TRUE").order("priority ASC, name ASC").each do |pa|
      hsh[pa.name] = { values: pa.values, selected: "selected", data_type: pa.data_type, mandatory: pa.mandatory}
    end
    hsh
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end

  # Callback Methods
  # -----------------

  def generate_and_save_code
    self.update_attribute(:code, SecureRandom.hex(4))
  end

  # Image Configuration
  # -------------
  def self.image_configuration
    {
      "Image::CoverImage" => {
        max_upload_limit: 10485760,
        min_upload_limit: 1,
        resolutions: [1000, 500],
        form_upload_image_label: "Upload a new Image",
        form_title: "Upload an Image",
        form_sub_title: "Please read the instructions below:",
        form_instructions: [
          "the filename should be in .jpg / .jpeg or .png format",
          "the image resolutions should be <strong>1000 x 500 Pixels</strong>",
          "the file size should be greater than 100 Kb and or lesser than <strong>10 MB</strong>",
          "Note: most cameras and camera phones will produce images bigger than this"
        ]
       }  
    }
    
  end
  
end
