class Dhatu::Service < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "services"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validate_string :name, mandatory: true, min_length: 2, format: /.*/i
  validate_string :short_description, mandatory: false, format: /.*/i
  validate_string :permalink, mandatory: true, format: /.*/i, min_length: 4, max_length: 128
  
  # Associations
  belongs_to :category, optional: true
  has_one :cover_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::CoverImage"
  has_many :gallery_images, :as => :imageable, :dependent => :destroy, :class_name => "Image::GalleryImage"

  # Meta Tag Associations
  has_one :og_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::OgImage"
  has_many :meta_tags, :as => :meta_taggable, :dependent => :destroy, :class_name => "Dhatu::MetaTag"

  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda {|query| where("LOWER(name) LIKE LOWER('%#{query}%') OR\
                                        LOWER(short_description) LIKE LOWER('%#{query}%') OR\
                                        LOWER(permalink) LIKE LOWER('%#{query}%') OR\
                                        LOWER(description) LIKE LOWER('%#{query}%')")
  }
  
  scope :filter_by_category, lambda { |c| where("category_id = ?", (c.is_a? Dhatu::Category ? c.id : c)) }

  scope :upcoming, lambda { where("created_at >= ?", Time.now) }
  scope :past, lambda { where("created_at < ?", Time.now) }

  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:name].to_s.strip.blank?

    service = Dhatu::Service.find_by_name(hsh[:name].to_s.strip) || Dhatu::Service.new
    service.name = hsh[:name].to_s.strip
    service.short_description = hsh[:short_description].to_s.strip
    service.generate_permalink
    service.description = hsh[:description].to_s.strip
    service.price = hsh[:price].to_s.strip
    service.duration = hsh[:duration].to_s.strip
    service.category = Dhatu::Category.find_by_name(hsh[:category].to_s.strip)
    service.status = hsh[:status].to_s.strip.blank? ? PUBLISHED : hsh[:status].to_s.strip
    service.featured = hsh[:featured].to_s.strip || true
    service.priority = hsh[:priority].to_s.strip || 1

    if service.valid?
      begin
        service.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving service: #{service.name}"
      details = "Error! #{service.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end

  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------
  def to_param
    "#{id}-#{name.parameterize[0..32]}"
  end

  def generate_permalink
    self.permalink = self.name.parameterize[0..32]
  end

  def display_name
    "#{name_was}"
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end

end