class Dhatu::TeamMember < Dhatu::ApplicationRecord

  # Set Table Name
  self.table_name = "team_members"

  # Including the State Machine Methods
  include Publishable
  include Featureable

  # Validations
  validates :name, presence: true, length: {minimum: 3, maximum: 256}, allow_blank: false
  validates :designation, presence: true, length: {minimum: 3, maximum: 256}, allow_blank: true
  validates :description, presence: true, length: {minimum: 3, maximum: 256}, allow_blank: true

  # Associations
  has_one :profile_image, :as => :imageable, :dependent => :destroy, :class_name => "Image::ProfileImage"

  # ------------------
  # Class Methods
  # ------------------

  scope :search, lambda {|query| where("
                                        LOWER(name) LIKE LOWER('%#{query}%') OR\
                                        LOWER(designation) LIKE LOWER('%#{query}%') OR\
                                        LOWER(status) LIKE LOWER('%#{query}%') OR\
                                        LOWER(linked_in_url) LIKE LOWER('%#{query}%') OR\
                                        LOWER(google_plus_url) LIKE LOWER('%#{query}%') OR\
                                        LOWER(facebook_url) LIKE LOWER('%#{query}%') OR\
                                        LOWER(twitter_url) LIKE LOWER('%#{query}%')")}
  
  scope :upcoming, lambda { where("created_at >= ?", Time.now) }
  scope :past, lambda { where("created_at < ?", Time.now) }

  def self.save_row_data(hsh)
    # Initializing error hash for displaying all errors altogether
    error_object = Kuppayam::Importer::ErrorHash.new

    return error_object if hsh[:name].to_s.strip.blank?

    team_member = Dhatu::TeamMember.find_by_name(hsh[:name].to_s.strip) || Dhatu::TeamMember.new
    team_member.name = hsh[:name].to_s.strip
    team_member.designation = hsh[:designation].to_s.strip
    team_member.description = hsh[:description].to_s.strip
    
    team_member.linked_in_url = hsh[:linked_in_url].to_s.strip
    team_member.google_plus_url = hsh[:google_plus_url].to_s.strip
    team_member.facebook_url = hsh[:facebook_url].to_s.strip
    team_member.twitter_url = hsh[:twitter_url].to_s.strip
    
    # team_member.category = Dhatu::Category.find_by_name(hsh[:category])
    team_member.status = hsh[:status].to_s.strip || PUBLISHED
    team_member.featured = hsh[:featured].to_s.strip || true
    team_member.priority = hsh[:priority].to_s.strip || 1

    if team_member.valid?
      begin
        team_member.save!
      rescue Exception => e
        summary = "uncaught #{e} exception while handling connection: #{e.message}"
        details = "Stack trace: #{e.backtrace.map {|l| "  #{l}\n"}.join}"
        error_object.errors << { summary: summary, details: details }        
      end
    else
      summary = "Error while saving team_member: #{team_member.name}"
      details = "Error! #{team_member.errors.full_messages.to_sentence}"
      error_object.errors << { summary: summary, details: details }
    end
    return error_object
  end

  # ------------------
  # Instance Methods
  # ------------------

  # Generic Methods
  # ---------------
  def to_param
    "#{id}-#{name.parameterize[0..32]}"
  end

  def display_name
    "#{name_was}"
  end

  # Permission Methods
  # ------------------

  def can_be_edited?
    status?(:published) or status?(:unpublished)
  end

  def can_be_deleted?
    status?(:removed)
  end

end